@@ -156,6 +156,9 @@ def n3map_main(argv):
         if options['zone_type'] == 'auto':
             options['zone_type'] = n3map.walker.detect_dnssec_type(zone,
                     qprovider, options['detection_attempts'])
+            if options['detect_only']:
+                print("{}: {}".format(str(zone), options['zone_type']))
+                return 0
 
         if options['zone_type'] == 'nsec3':
             (hash_queues, process_pool) = prehash.create_prehash_pool(
@@ -222,7 +225,8 @@ def n3map_main(argv):
                                          startname=options['start'],
                                          endname=options['end'],
                                          stats=stats,
-                                         output_file=output_rrfile)
+                                         output_file=output_rrfile,
+                                         never_prefix_label=options['no_prefix_labels'])
             elif options['query_mode'] == "A":
                 walker = NSECWalkerA(zone,
                                      qprovider,
@@ -231,7 +235,8 @@ def n3map_main(argv):
                                      startname=options['start'],
                                      endname=options['end'],
                                      stats=stats,
-                                     output_file=output_rrfile)
+                                     output_file=output_rrfile,
+                                     never_prefix_label=options['no_prefix_labels'])
             else:
                 walker = NSECWalkerN(zone,
                                      qprovider,
@@ -283,6 +288,7 @@ def default_options():
             'query_chars' : 'binary',
             'start' : None,
             'end' : None,
+            'no_prefix_labels' : False,
             'label_counter' : None,
             'hashlimit' : 0,
             'timeout' : 2500,
@@ -298,6 +304,7 @@ def default_options():
             'queue_element_size' : 256,
             'use_openssl' : True,
             'ipproto' : '',
+            'detect_only' : False,
             }
     return opts
 
@@ -333,11 +340,13 @@ def parse_arguments(argv):
             'queue-element-size=',
             'quiet',
             'start=',
+            'no-prefix-labels',
             'timeout=',
             'no-openssl',
             'verbose',
             'color=',
-            'version'
+            'version',
+            'detect-only'
     ]
     options = default_options()
     opts = '346AMNabc:e:f:hi:lm:no:pqs:v'
@@ -362,6 +371,9 @@ def parse_arguments(argv):
         elif opt in ('-3', '--nsec3'):
             options['zone_type'] = 'nsec3'
 
+        elif opt in ('--detect-only'):
+            options['detect_only'] = True
+
         elif opt in ('-4',):
             options['ipproto'] = 'ipv4'
 
@@ -414,6 +426,9 @@ def parse_arguments(argv):
         elif opt in ('-b', '--binary'):
             options['query_chars'] = 'binary'
 
+        elif opt in ('--no-prefix-labels',):
+            options['no_prefix_labels'] = True
+
         elif opt in ('-e', '--end'):
             options['end'] = arg
 
@@ -577,6 +592,13 @@ NSEC Options:
   -e, --end=DOMAIN           use DOMAIN as the enumeration start-/endpoint.
                                DOMAIN is relative to the zone name.
 
+      --no-prefix-labels     do not add leading labels ("\\x00.") to increment
+                               query names. Applies to 'A' query mode.
+                               This will result in incomplete enumeration for
+                               many zones, but is useful to avoid descending
+                               into subzones when querying non-authoritative
+                               namservers or to speed up the enumeration of TLDs.
+
 NSEC3 Options:
   -f, --aggressive=N         send up to N queries in parallel. This may speed
                                up the enumeration significantly if the DNS
@@ -603,7 +625,8 @@ Advanced NSEC3 Options:
   Use with caution.
       --label-counter=N      set the initial label counter
       --queue-element-size=N set the queue elment size. (default {queue_element_sz:d})
-      --no-openssl           do not use openssl for hashing (slower)
+      --no-openssl           do not use OpenSSL for hashing (slower, especially
+                              at high iteration counts)
 
 General Options:
   -q, --quiet                do not display progress information during enumeration
@@ -621,6 +644,8 @@ General Options:
       --detection-attempts=N limit the maximum number of zone type (NSEC/NSEC3)
                                detection attempts. N=0 specifies no limit.
                                (default {detection_attempts:d})
+      --detect-only          detect and print zone type only, don't enumerate
+                               the zone.
       --omit-soa-check       don't check the SOA record of the zone
                                before starting enumeration (use with caution).
       --omit-dnskey-check    don't check the DNSKEY record of the zone
